<?php

// phpcs:disable BestIt.DocTags.RequiredPropertyTags.TagOccurrenceMinVar
// phpcs:disable BestIt.TypeHints.PropertyTypeHint.MissingAnyTypeHint
// phpcs:disable BestIt.TypeHints.PropertyTypeHint.MissingNativeTypeHint

namespace App\Console\Commands;

use App\Console\CrawlerHelper;
use Atymic\Twitter\Facade\Twitter;
use Exception;
use Illuminate\Console\Command;

/**
 * Twitter Unfollower
 *
 * Unfollow all the following accounts for our account (used on API)
 *
 * @author Lucas Buzzo <lucas.buzzo@ab-inbev.com>
 * @package App\Console\Commands
 */

class TwitterUnfollow extends Command
{
    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Unfollow everyone from our Twitter\'s account';

    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'twitter:unfollow';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return int
     */
    public function handle(): int
    {
        $this->line('Running Twitter Unfollower');

        app(CrawlerHelper::class)->sendNotification('Twitter: Unfollowing Accounts');

        try {
            $friends = Twitter::getFriends();

            foreach ($friends->users as $friend) {
                // Get a number between 5 and 10 to be used at usleep and avoid a bot behavior
                $random = rand(5, 10);

                Twitter::postUnfollow(['screen_name' => $friend->screen_name]);

                $this->line('Twitter: unfollowed @' . $friend->screen_name);

                // The range will be 500ms to 1 second
                usleep($random * 100);
            }
            $return = 0;
        } catch (Exception $e) {
            $this->error('Failed to unfollow accounts at Twitter');
            $this->error($e->getMessage());

            app(CrawlerHelper::class)->sendNotification('Twitter: Failed when unfollowing accounts. Error message: ' . $e->getMessage());

            $return = 1;
        }

        return $return;
    }
}
