/* eslint-disable promise/param-names */
/* eslint-disable no-void */
const chromium = require('@sparticuz/chromium');

const { addExtra } = require('puppeteer-extra');
const puppeteer = require('puppeteer-core');
const puppeteerExtra = addExtra(puppeteer);

const anonymizeUa = require('puppeteer-extra-plugin-anonymize-ua')(); // ^2.4.4
puppeteerExtra.use(anonymizeUa);

const lighthouse = require('lighthouse'); // "^9.2.0",

exports.run = async function (req, res) {
  let link = req.query.link || (req.body && req.body.link);

  /*
  // Dropped because we'll generate the full report
  const skipAudits = [
    'first-contentful-paint',
    'largest-contentful-paint',
    'first-meaningful-paint',
    'speed-index',
    'max-potential-fid',
    'cumulative-layout-shift',
    'interactive',
    'total-blocking-time',
    'max-potential-fid',
    'uses-http2',
  ];
  */

  let final = {};
  const executions = [];
  let responseCode = 0;

  const browser = await puppeteerExtra.launch({
    headless: true,
    ignoreHTTPSErrors: true,
    args: [
      // '--disable-gpu',
      // '--disable-dev-shm-usage',
      // '--disable-setuid-sandbox',
      '--no-first-run',
      '--no-sandbox',
      // '--no-zygote',
      // '--single-process',
      "--proxy-server='direct://'",
      '--proxy-bypass-list=*',
      '--disable-extensions',
      '--disable-default-apps',
      '--disable-component-extensions-with-background-pages',
    ],
    executablePath: await chromium.executablePath,
    defaultViewport: chromium.defaultViewport,
  });

  const page = await browser.newPage();

  // Increase the default time limit (2 minutes)
  await page.setDefaultNavigationTimeout(120000);

  console.log('Starting: ' + link);

  try {
    let response = [];

    // We'll use promise race to ensure page dont exced our timeout (1.5 minute)
    response = await Promise.race([
      page.goto(link).catch((e) => void e),
      new Promise((x) => setTimeout(x, 90 * 1000)),
    ]);

    if (response === undefined || response.status() >= 400) {
      // Let's try to open with a https - some links isnt redirecting properly
      link = link.replace('http://', 'https://');

      response = await Promise.race([
        page.goto(link).catch((e) => void e),
        new Promise((x) => setTimeout(x, 115 * 1000)),
      ]);

      await page.waitForTimeout(10000);

      if (response === undefined) {
        throw new Error('Unexpected problem with the link');
      }

      // Due to the change from waiting networkIdle to DOMContentLoadade, additional timeout
      await page.waitForTimeout(15000);
    }

    if (response && response.status() >= 400) {
      final.siteStatus = 'offline';

      throw new Error('Invalid HTTP Status');
    }

    const linkFinal = page.url();

    /*
    const finalUrl = new URL(linkFinal);

    const initialUrl = new URL(link);

    if (
      finalUrl.hostname.replace('www.', '') !==
      initialUrl.hostname.replace('www.', '')
    ) {
      throw new Error('Redirect deteted');
    }
    */

    await page.waitForTimeout(2000);

    await page.goto('about:blank');

    const interactions = new Array(5).fill(0);

    let finalLhr = null;

    for await (const i of interactions) {
      // eslint-disable-next-line no-undef
      const execution = ({ lhr } = await lighthouse(linkFinal, {
        port: new URL(browser.wsEndpoint()).port,
        output: 'json',
        logLevel: 'silent',
        chromeFlags: ['--disable-storage-reset'],
        onlyCategories: ['performance'],
      })).lhr;

      console.log(
        'Executed with score: ' + execution.categories.performance.score * 100
      );

      if (
        !finalLhr ||
        execution.categories.performance.score >=
          finalLhr.categories.performance.score
      ) {
        finalLhr = execution;
      }

      executions.push({
        interaction: i,
        score: execution.categories.performance.score * 100,
      });

      await page.waitForTimeout(2000);
    }

    // In case of success, final will be the better lighthouse execution
    final = finalLhr;

    console.log(
      'Finished with score: ' + finalLhr.categories.performance.score * 100
    );

    /*
    // Dropped because we'll generate the full report
    const problems = [];
    Object.values(lhr.audits).forEach((audit) => {
      if (audit.score !== null && audit.score < 1) {
        if (!skipAudits.includes(audit.id)) {
          problems.push({
            title: audit.title,
            description: audit.description,
            impact: audit.displayValue ? audit.displayValue : 'undefined',
          });
        }
      }
    });
    */

    // final.problems = problems;
    // final.status = 'success';
    responseCode = 200;
  } catch (error) {
    responseCode = 500;

    // In case of fail, our final can store informations to help to debug
    final.status = 'failed';
    final.error = error.message;
    final.line = error.line;

    console.log('Error message: ' + error.message);
  } finally {
    await browser.close();

    console.log('Finished with status: ' + responseCode);

    final = JSON.stringify(final);

    res.status(responseCode).send(final);

    // res.status(responseCode).send(executions);

    // eslint-disable-next-line no-unsafe-finally
    return;
  }
};
