/* eslint-disable promise/param-names */
/* eslint-disable no-void */
/* eslint-disable new-cap */
/* eslint-disable object-curly-spacing */
/* eslint-disable indent */
/* eslint-disable max-len */

const { Pool } = require('pg');

const chromium = require('@sparticuz/chromium');
require('dotenv').config();

const { addExtra } = require('puppeteer-extra');
const puppeteer = require('puppeteer-core');
const puppeteerExtra = addExtra(puppeteer);

const stealthPlugin = require('puppeteer-extra-plugin-stealth');
puppeteerExtra.use(stealthPlugin());

const [, , ...args] = process.argv;

async function psqlConnect() {
  if (global.connection) {
    return global.connection.connect();
  }

  try {
    const username = process.env.DB2_USERNAME;
    const password = process.env.DB2_PASSWORD;
    const host = process.env.DB2_HOST;
    const database = process.env.DB2_DATABASE;

    const pool = new Pool({
      connectionString: `postgres://${username}:${password}@${host}:5432/${database}`,
    });

    global.connection = pool;
    return pool.connect();
  } catch (error) {
    console.log(
      'Connection with database failed. Error message: ' + error.message
    );

    return null;
  }
}

async function deleteOldSources(link) {
  try {
    const db = await psqlConnect();

    await db.query(
      `DELETE FROM crawls where (data_type = 'crawler:htmlwithjs' or data_type = 'crawler:browsersrequest') and link = '${link}';`
    );

    await db.query(
      `DELETE FROM crawls_failed where (data_type = 'crawler:htmlwithjs' or data_type = 'crawler:browsersrequest') and link = '${link}';`
    );

    db.release();

    return 0;
  } catch (error) {
    console.log(
      'Failed when deleting old Crawls entries for: ' +
        link +
        '. Error message: ' +
        error.message
    );

    return null;
  }
}

async function storeData(
  link,
  executedAt,
  content,
  official,
  failed,
  dataType
) {
  try {
    const db = await psqlConnect();

    const table = failed ? 'crawls_failed' : 'crawls';

    await db.query(
      `INSERT INTO ${table} (link, data_type, content, executed_at, official) VALUES ($1, $2, $3, $4, $5)`,
      [link, dataType, content, executedAt, official]
    );

    await db.release();
  } catch (error) {
    console.log(error.message);
  } finally {
    // eslint-disable-next-line no-unsafe-finally
    return 1;
  }
}

/**
 * Run test
 *
 * @param {string} link
 * @param {string} additional
 */
async function test(
  link,
  originalLink,
  official,
  executedAt,
  additional = '{}'
) {
  // console.log(await chromeLambda.executablePath);
  // return;
  let ageGateBypass = null;
  try {
    ageGateBypass = JSON.parse(additional).agegate;
  } catch {
    ageGateBypass = null;
  }

  let agegateStorage = '';
  let agegateData = '';

  if (ageGateBypass) {
    agegateStorage = ageGateBypass.storage;
    agegateData = ageGateBypass.data;
  }

  const final = {};

  const browser = await puppeteerExtra.launch({
    headless: true,
    ignoreHTTPSErrors: true,
    args: [
      '--no-sandbox',
      '--disable-setuid-sandbox',
      '--window-size=1920,1080',
      '--lang=pt-BR,pt',
      '--disable-sync',
      '--disable-translate',
      '--no-first-run',
      '--safebrowsing-disable-auto-update',
      '--disable-prompt-on-repost',
      '--disable-popup-blocking',
      '--disable-default-apps',
      '--password-store=basic',
      '--use-mock-keychain',
      '--disable-crash-reporter',
      "--proxy-server='direct://'",
      '--proxy-bypass-list=*',
      '--deterministic-fetch',
    ],
    executablePath: await chromium.executablePath,
    defaultViewport: chromium.defaultViewport,
  });

  const page = await browser.newPage();

  // Increase the default timeout limit (2 minutes)
  await page.setDefaultNavigationTimeout(120000);

  // We need to intercept the requests to store
  const requestList = {
    allowAll: [],
    initial: [],
  };

  let allowedAllCookies = false;

  await page.setRequestInterception(true);

  page.on('request', (request) => {
    if (allowedAllCookies) {
      requestList.allowAll.push(request.url());
    } else {
      requestList.initial.push(request.url());
    }

    request.continue();
  });

  let sourceCode = '';

  try {
    let response = [];
    // We'll use promise race to ensure page dont exced our timeout (1.5 minute)

    response = await Promise.race([
      page.goto(link, { waitUntil: 'domcontentloaded' }).catch((e) => void e),
      new Promise((x) => setTimeout(x, 115 * 1000)),
    ]);

    await page.waitForTimeout(10000);

    if (response === undefined || response.status() >= 400) {
      // Let's try to open with a https - some links isnt redirecting properly
      link = link.replace('http://', 'https://');

      response = await Promise.race([
        page.goto(link, { waitUntil: 'domcontentloaded' }).catch((e) => void e),
        new Promise((x) => setTimeout(x, 115 * 1000)),
      ]);

      await page.waitForTimeout(10000);

      if (response === undefined) {
        throw new Error('Unexpected problem with the link');
      }

      // Due to the change from waiting networkIdle to DOMContentLoadade, additional timeout
      await page.waitForTimeout(15000);
    }

    if (response && response.status() >= 400) {
      final.onetrustImplemented = false;
      final.siteStatus = 'offline';

      throw new Error('Invalid HTTP Status');
    }

    sourceCode = await page.content();

    const linkFinal = page.url();

    const finalUrl = new URL(linkFinal);

    const initialUrl = new URL(link);

    const social =
      initialUrl.hostname.includes('facebook.com') ||
      initialUrl.hostname.includes('instagram.com') ||
      initialUrl.hostname.includes('youtube.com');

    if (
      !social &&
      finalUrl.hostname.replace('www.', '').toLowerCase() !==
        initialUrl.hostname.replace('www.', '').toLowerCase()
    ) {
      throw new Error('Redirect deteted');
    }

    // Used to ensure, we'll not get same iframe twice
    const iframeList = [];

    for (const frame of page.frames()) {
      const iframeUrl = await frame.url();

      if (
        iframeUrl &&
        iframeUrl.length > 3 &&
        iframeUrl.indexOf('.') !== -1 &&
        !iframeList.includes(iframeUrl)
      ) {
        const domainIframe = new URL(iframeUrl);

        if (
          domainIframe &&
          domainIframe.hostname.replace('www.', '') !==
            initialUrl.hostname.replace('www.', '')
        ) {
          sourceCode += '\n\n' + (await frame.content());
          iframeList.push(iframeUrl);
        }
      }
    }

    if (agegateData.length > 0) {
      switch (agegateStorage) {
        case 'cookie':
        case 'cookies':
          await page.evaluate((agegateData) => {
            const d = new Date();
            d.setTime(d.getTime() + 30 * 24 * 60 * 60 * 1000);
            const expires = 'expires=' + d.toUTCString();

            agegateData.forEach((ageData) => {
              document.cookie =
                ageData.key + '=' + ageData.value + ';' + expires + ';path=/';
            });
          }, agegateData);

          break;

        case 'session_storage':
          await page.evaluate((agegateData) => {
            agegateData.forEach((ageData) => {
              sessionStorage.setItem(ageData.key, ageData.value);
            });
          }, agegateData);
          break;

        case 'local_storage':
          await page.evaluate((agegateData) => {
            agegateData.forEach((ageData) => {
              localStorage.setItem(ageData.key, ageData.value);
            });
          }, agegateData);
          break;

        case 'redirect':
          await Promise.race([
            page
              .goto(agegateData[0].value, { waitUntil: 'domcontentloaded' })
              .catch((e) => void e),
            new Promise((x) => setTimeout(x, 60 * 1000)),
          ]);
          break;

        case 'steps':
          for await (const step of agegateData) {
            switch (step.action) {
              case 'fill':
                await page.waitForSelector(step.selector, {
                  visible: true,
                  timeout: 3000,
                });
                await page.focus(step.selector);
                await page.keyboard.type(step.value, { delay: 200 });
                break;
              case 'click':
                await page.waitForSelector(step.selector);
                await page.click(step.selector);
                break;
              case 'select':
                await page.waitForSelector(step.selector, {
                  visible: true,
                  timeout: 3000,
                });
                await page.select(step.selector, step.value);
                await page.waitForTimeout(1000);
                break;
              case 'end':
                await page.waitForTimeout(5000);
                break;
              default:
                break;
            }
          }
          break;
        default:
          break;
      }
    }

    await Promise.race([
      page.goto('about:blank').catch((e) => void e),
      new Promise((x) => setTimeout(x, 5 * 1000)),
    ]);

    // We only need to go again to the website, when:
    // Bypass age-gate or need network requests
    // We'll use promise race to ensure page dont exced our timeout (1.5 minute)
    await Promise.race([
      page.goto(link, { waitUntil: 'domcontentloaded' }).catch((e) => void e),
      new Promise((x) => setTimeout(x, 115 * 1000)),
    ]);

    await page.waitForTimeout(10000);

    if (agegateData.length > 0) {
      sourceCode += '\n\n' + (await page.content());

      // Use this to get the content from iframes inside the page
      for (const frame of page.frames()) {
        const iframeUrl = await frame.url();

        if (
          iframeUrl &&
          iframeUrl.length > 3 &&
          iframeUrl.indexOf('.') !== -1 &&
          !iframeList.includes(iframeUrl)
        ) {
          const domainIframe = new URL(iframeUrl);

          if (
            domainIframe &&
            domainIframe.hostname.replace('www.', '') !==
              initialUrl.hostname.replace('www.', '')
          ) {
            sourceCode += '\n\n' + (await frame.content());
            iframeList.push(iframeUrl);
          }
        }
      }
    }

    await page.waitForTimeout(5000);

    allowedAllCookies = true;

    await page.evaluate(() => {
      if (typeof Optanon !== 'undefined') {
        try {
          // eslint-disable-next-line no-undef
          Optanon.AllowAll();
        } catch (error) {}
      }
    });

    await page.waitForTimeout(10000);

    final.requests = requestList;

    final.sourceCode = sourceCode;

    let innerText = '';

    innerText = await page.evaluate(() => {
      return document.querySelector('body').innerText;
    });
    final.sourceCode += '\n\n' + innerText;

    await deleteOldSources(originalLink);

    await storeData(
      originalLink,
      executedAt,
      final.sourceCode,
      official,
      false,
      'crawler:htmlwithjs'
    );

    await storeData(
      originalLink,
      executedAt,
      final.requests,
      official,
      false,
      'crawler:browsersrequest'
    );

    final.status = 'success';
  } catch (error) {
    final.status = 'failed';
    final.error = error.message;
    final.sourceCode = '';

    await deleteOldSources(originalLink);

    await storeData(
      originalLink,
      executedAt,
      error.message,
      official,
      true,
      'crawler:htmlwithjs'
    );

    await storeData(
      originalLink,
      executedAt,
      error.message,
      official,
      true,
      'crawler:browsersrequest'
    );
  } finally {
    const finalCoded = JSON.stringify({ status: final.status });
    console.log(finalCoded);

    browser.close();

    if (final.status === 'success') {
      process.exit();
    } else {
      process.exit(1);
    }
  }
}

const link = args[0];
const originalLink = args[1];
const official = args[2];
const executedAt = args[3];
const additional = args[4];

test(link, originalLink, official, executedAt, additional);
