<?php
// phpcs:disable PSR2.ControlStructures.ElseIfDeclaration.NotAllowed

namespace App\Http\Controllers\Helpers;

use App\Http\Controllers\Controller;
use DateTime;
use Illuminate\Http\Request;

/**
 * HelpersController Controller
 *
 * @author Tércio Lacerda <Tercio.lacerda-ext@ab-inbev.com>
 * @package App\Http\Controllers\Helpers
 */
class HelpersController extends Controller
{
  /**
   * Build request's where
   *
   * @param Object $query
   * @param Array $params
   * @param Array $bdMapColumns
   * @param String $type
   *
   * @return void
   */
  public static function buildRequestWhere(Object $query, array $params, array $bdMapColumns, String $type = ''): void
  {
    if ($type === 'export-ticket-asset-details') {
      $params['ticketType'] = array(json_encode(array('value' => 'asset-details-ticket')));
      $params['ticketStatus'] = array(json_encode(array('value' => 'approved')));
    }

    if ($type === 'website-data' || $type === 'excelence' || $type === 'privacy-and-security' || $type === 'google-analytics') {
      $params['channels'] = array(json_encode(array('value' => 'Site')));
    }

    foreach ($params as $key => $value) {
      if ($key === 'page' || $key === 'per_page' || $key === 'approved' || $value === null || $value === '' || empty($bdMapColumns[$key])) continue;

      // If is from a multiple select in front-end, always get item value property
      if (gettype($value) === 'array') {
        $query->where(function ($queryX) use ($value, $bdMapColumns, $key, $params) {
          foreach ($value as $keyArray => $valueArray) {
            $jsonArrayValue = null;
            if (is_array($valueArray)) {
              $jsonArrayValue = $valueArray['value'];
            } else {
              $jsonArrayValue = json_decode($valueArray, true)['value'];
            }

            if ($key === 'criteria') {
              if (empty($bdMapColumns[$jsonArrayValue])) continue;
              $approved = (!(empty($params['approved'])) && $params['approved'] === 'true') ? true : false;

              if ($jsonArrayValue === 'lighthouse_last' || $jsonArrayValue === 'website_data' || $jsonArrayValue === 'semrush' || $jsonArrayValue === 'lighthouse_avg' || $jsonArrayValue === 'property_excellence' || $jsonArrayValue === 'privacy_security') {
                $referenceValue = 0;
                $operator = '<';
                if ($jsonArrayValue === 'lighthouse_last') $referenceValue = 50;
                if ($jsonArrayValue === 'website_data') $referenceValue = 90;
                if ($jsonArrayValue === 'semrush') $referenceValue = 75;
                if ($jsonArrayValue === 'lighthouse_avg') $referenceValue = 50;
                if ($jsonArrayValue === 'property_excellence') $referenceValue = 100;
                if ($jsonArrayValue === 'privacy_security') $referenceValue = 100;
                if ($approved) $operator = '>=';
                $queryX->where(function ($queryY) use ($bdMapColumns, $jsonArrayValue, $operator, $referenceValue) {
                  $queryY->where($bdMapColumns[$jsonArrayValue], $operator, $referenceValue);
                });
              } elseif ($jsonArrayValue === 'risk_rate' || $jsonArrayValue === 'vulnerability_level') {
                $queryX->where(function ($queryY) use ($bdMapColumns, $jsonArrayValue, $approved) {
                  if ($approved) {
                    $queryY->where($bdMapColumns[$jsonArrayValue], 'ILIKE', 'Low');
                    $queryY->orWhere($bdMapColumns[$jsonArrayValue], 'ILIKE', 'Medium');
                  } else {
                    $queryY->where($bdMapColumns[$jsonArrayValue], 'ILIKE', 'High');
                  }
                });
              } elseif ($jsonArrayValue === 'pia_completion') {
                $queryX->where(function ($queryY) use ($bdMapColumns, $jsonArrayValue, $approved) {
                  if ($approved) {
                    $queryY->where($bdMapColumns[$jsonArrayValue], 'ILIKE', 'Completed');
                  } else {
                    $queryY->whereNotIn($bdMapColumns[$jsonArrayValue], array('Completed', 'completed'));
                  }
                });
              } elseif ($jsonArrayValue === 'age_gate' || $jsonArrayValue === 'coi' || $jsonArrayValue === 'fan' || $jsonArrayValue === 'rdm' || $jsonArrayValue === 'ugc') {
                // Boolean string criterias
                $queryX->where(function ($queryY) use ($bdMapColumns, $jsonArrayValue, $approved) {
                  $queryY->where($bdMapColumns[$jsonArrayValue], '=', $approved ? 'true' : 'false');
                });
              } else {
                // Boolean criterias
                $queryX->where(function ($queryY) use ($bdMapColumns, $jsonArrayValue, $approved) {
                  $queryY->where($bdMapColumns[$jsonArrayValue], '=', $approved);
                });
              }
            } elseif ($key === 'lighthouse') {
              $operator = '<';
              $referenceValue = 50;
              if ($jsonArrayValue === true || $jsonArrayValue === 'true') {
                $operator = '>=';
              }
              $queryX->orWhere($bdMapColumns[$key], $operator, $referenceValue);
            } else {
              $queryX->orWhere($bdMapColumns[$key], '=', $jsonArrayValue);
            }
          }
        });
      } else {
        $json = json_decode($value, true);
        if ($json) {
          if ($key === 'dateRange' && $json['start'] && $json['end']) {
            $start = new DateTime($json['start']);
            $end = new DateTime($json['end']);
            $start->setTime(0, 0, 0);
            $end->setTime(23, 59, 59);
            $query->where(function ($queryX) use ($bdMapColumns, $key, $start, $end) {
              $queryX->whereBetween($bdMapColumns[$key], array($start->format('Y-m-d H:i:s'), $end->format('Y-m-d H:i:s')));
            });
          }
        } else {
          $query->where(function ($queryX) use ($value, $bdMapColumns, $key) {
            if ($key === 'date') {
              $queryX->whereDate($bdMapColumns[$key], $value);
              return;
            }
            $operator = '=';
            if ($key === 'text' || $key === 'email') {
              $operator = 'ILIKE';
              $value = "%$value%";
            }
            $queryX->where($bdMapColumns[$key], $operator, $value);
          });
        }
      }
    }
  }

  /**
   * Mount where raw
   *
   * @param Request $request
   * @param String $type
   *
   * @return String
   */
  public static function mountWhereRaw(Request $request, String $type = ''): String
  {
    // -- Filter conditions
    $conditions = array();
    $zones = $request->zones;
    $countries = $request->countries;
    $brands = $request->brands;
    $channels = $request->channels;
    $departments = $request->department;
    $text = $request->text;
    $criteria = $request->criteria;
    $approved = $request->approved;
    $lighthouse = $request->lighthouse;

    if ($zones && count($zones) > 0) {
      foreach ($zones as $key => $value) {
        array_push($conditions, ['zone', '=', $value['value'], 'zones']);
      }
    }

    if ($countries && count($countries) > 0) {
      foreach ($countries as $key => $value) {
        array_push($conditions, ['country', '=', $value['value'], 'countries']);
      }
    }

    if ($brands && count($brands) > 0) {
      foreach ($brands as $key => $value) {
        array_push($conditions, ['brand', '=', $value['text'], 'brands']);
      }
    }

    if ($channels && count($channels) > 0) {
      foreach ($channels as $key => $value) {
        array_push($conditions, ['channel', '=', $value['text'], 'channels']);
      }
    }

    if ((strlen($type) > 0) && ($type === 'page-speed' || $type === 'website-data' || $type === 'excelence' || $type === 'privacy-and-security' || $type === 'google-analytics')) array_push($conditions, ['channel', 'ILIKE', 'site', 'channels']);

    if ($departments && count($departments) > 0) {
      foreach ($departments as $key => $value) {
        array_push($conditions, ['department', '=', $value['value'], 'departments']);
      }
    }

    if ($text && strlen($text) > 0) {
      array_push($conditions, ['link', 'ILIKE', '%' . $text . '%', 'links']);
    }

    if (($criteria && count($criteria) > 0) && !empty($approved)) {
      foreach ($criteria as $key => $value) {
        $valueField = $value['value'];
        if ($valueField === 'lighthouse_last' || $valueField === 'website_data' || $valueField === 'semrush' || $valueField === 'lighthouse_avg' || $valueField === 'property_excellence' || $valueField === 'privacy_security') {
          $number = 0;
          if ($valueField === 'lighthouse_last') $number = 50;
          if ($valueField === 'website_data') $number = 90;
          if ($valueField === 'semrush') $number = 75;
          if ($valueField === 'lighthouse_avg') $number = 50;
          if ($valueField === 'property_excellence') $number = 100;
          if ($valueField === 'privacy_security') $number = 100;
          if ($approved === 'true') {
            array_push($conditions, [$valueField, '>=', $number, $valueField]);
          } else {
            array_push($conditions, [$valueField, '<', $number, $valueField]);
          }
        } elseif ($valueField === 'risk_rate' || $valueField === 'vulnerability_level') {
          if ($approved === 'true') {
            array_push($conditions, [$valueField, 'ILIKE', 'Low', $valueField]);
            array_push($conditions, [$valueField, 'ILIKE', 'Medium', $valueField]);
          } else {
            array_push($conditions, [$valueField, 'ILIKE', 'High', $valueField]);
          }
        } elseif ($valueField === 'pia_completion') {
          if ($approved === 'true') {
            array_push($conditions, [$valueField, 'ILIKE', 'Completed', $valueField]);
          } else {
            array_push($conditions, [$valueField, '!=', 'completed', $valueField]);
          }
        } else { // "Booleans strings"
          array_push($conditions, [$valueField, '=', "$approved", $valueField]);
        }
      }
    }

    if ($type === 'page-speed') {
      if ($lighthouse && count($lighthouse) > 0) {
        foreach ($lighthouse as $key => $value) {
          $value = json_decode($value, true);
          if ($value['value'] === true || $value['value'] === 'true') {
            array_push($conditions, ['lighthouse_last', '>=', 50, 'lighthouse_last']);
          }
          if ($value['value'] === false || $value['value'] === 'false') {
            array_push($conditions, ['lighthouse_last', '<', 50, 'lighthouse_last']);
          }
        }
      }
    }

    $lastFilter = '';
    $whereRaw = '';
    $lastCondition = count($conditions) - 1;
    foreach ($conditions as $key => $condition) {
      if ($key === 0) {
        $lastFilter = $condition[3];
        $whereRaw = "($condition[0] $condition[1] '$condition[2]'";
        if ($key === $lastCondition) $whereRaw = $whereRaw . ')';
        continue;
      }
      if ($lastFilter === $condition[3]) {
        $whereRaw = $whereRaw . " OR $condition[0] $condition[1] '$condition[2]'";
        if ($key === $lastCondition) $whereRaw = $whereRaw . ')';
      } else {
        $lastFilter = $condition[3];
        $whereRaw = $whereRaw . ") AND ($condition[0] $condition[1] '$condition[2]'";
        if ($key === $lastCondition) $whereRaw = $whereRaw . ')';
      }
    }
    return $whereRaw;
  }
}
