<?php

// phpcs:disable BestIt.DocTags.RequiredPropertyTags.TagOccurrenceMinVar
// phpcs:disable BestIt.TypeHints.PropertyTypeHint.MissingAnyTypeHint

namespace App\Models;

use GeneaLabs\LaravelModelCaching\Traits\Cachable;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\SoftDeletes;

/**
 * Country Class
 *
 * Specifies the fillable atrributes and base configation
 *
 * @author Lucas Buzzo <lucas.buzzo@ab-inbev.com>
 * @package App\Models
 */

class Country extends Model
{
    use Cachable;
    use HasFactory;
    use SoftDeletes;

    /**
     * The attributes that are converted to Carbon object
     */
    protected $dates = ['deleted_at'];

    /**
     * The attributes that are mass assignable.
     */
    protected $fillable = [
        'code',
        'locale_id',
        'name',
        'zone_id',
    ];

    /**
     * Define the table name
     */
    protected $table = 'countries';

    /**
     * Return the Links
     *
     * @return HasMany
     */
    public function links(): HasMany
    {
        return $this->hasMany(Links::class, 'country_id', 'id');
    }

    /**
     * Return the locale
     *
     * @return BelongsTo
     */
    public function locale(): BelongsTo
    {
        return $this->belongsTo(Locale::class, 'locale_id');
    }

    /**
     * Return the Zone
     *
     * @return BelongsTo
     */
    public function zone(): BelongsTo
    {
        return $this->belongsTo(Zone::class, 'zone_id');
    }
}
