<?php

namespace App\Models;

use GeneaLabs\LaravelModelCaching\Traits\Cachable;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\SoftDeletes;

/**
 * Keywords Class
 *
 * Specifies the fillable atrributes and base configation
 *
 * @author Lucas Buzzo <lucas.buzzo@ab-inbev.com>
 * @package App\Models
 */
class Keywords extends Model
{
    use Cachable;
    use SoftDeletes;

    /**
     * The attributes that are mass assignable.
     */
    protected $fillable = [
        'keyword',
        'criteria_id',
        'locale_id',
        'social',
        'group',
    ];

    /**
     * Define the table name
     */
    protected $table = 'keywords';

    /**
     * Return the criteria
     *
     * @return BelongsTo
     */
    public function criteria(): BelongsTo
    {
        return $this->belongsTo(Criteria::class, 'criteria_id');
    }

    /**
     * Return the criteria
     *
     * @return BelongsTo
     */
    public function locale(): BelongsTo
    {
        return $this->belongsTo(Locale::class, 'locale_id');
    }
}
