<?php

// phpcs:disable BestIt.DocTags.RequiredPropertyTags.TagOccurrenceMinVar
// phpcs:disable BestIt.TypeHints.PropertyTypeHint.MissingAnyTypeHint

namespace App\Models;

use GeneaLabs\LaravelModelCaching\Traits\Cachable;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\SoftDeletes;

/**
 * LinkCriteria Class
 *
 * Specifies the fillable atrributes and base configation
 *
 * @author Lucas Buzzo <lucas.buzzo@ab-inbev.com>
 * @package App\Models
 */

class LinkCriteria extends Model
{
    use Cachable;
    use HasFactory;
    use SoftDeletes;

    /**
     * The attributes that are mass assignable.
     */
    protected $fillable = [
        'criteria_id',
        'link_id',
        'executed_at',
        'response',
        'details',
        'official',
    ];

    /**
     * Define the table name
     */
    protected $table = 'links_criteria';

    /**
     * Return the Criteria
     *
     * @return BelongsTo
     */
    public function criteria(): BelongsTo
    {
        return $this->belongsTo(Criteria::class, 'criteria_id');
    }

    /**
     * Return the Link
     *
     * @return BelongsTo
     */
    public function link(): BelongsTo
    {
        return $this->belongsTo(Links::class, 'link_id');
    }
}
