<?php

// phpcs:disable BestIt.DocTags.RequiredPropertyTags.TagOccurrenceMinVar
// phpcs:disable BestIt.TypeHints.PropertyTypeHint.MissingAnyTypeHint

namespace App\Models;

use GeneaLabs\LaravelModelCaching\Traits\Cachable;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\SoftDeletes;

/**
 * Links Class
 *
 * Specifies the fillable atrributes and base configation
 *
 * @author Lucas Buzzo <lucas.buzzo@ab-inbev.com>
 * @package App\Models
 */

class Links extends Model
{
    use Cachable;
    use HasFactory;
    use SoftDeletes;

    /**
     * The attributes that are mass assignable.
     */
    protected $fillable = [
        'link',
        'additional_crawler',
        'semrush_project_id',
        'brand_id',
        'channel_id',
        'country_id',
        'owner_id',
        'http_status',
        'business_area',
        'environment',
        'consider',
        'scope',
        'in_ga',
        'in_acquia',
        'in_bp',
        'in_cdp',
        'alcoholic',
    ];

    /**
     * Define the table name
     */
    protected $table = 'links';

    /**
     * Return the Brand
     *
     * @return BelongsTo
     */
    public function brand(): BelongsTo
    {
        return $this->belongsTo(Brand::class, 'brand_id');
    }

    /**
     * Return the Channel
     *
     * @return BelongsTo
     */
    public function channel(): BelongsTo
    {
        return $this->belongsTo(Channel::class, 'channel_id');
    }

    /**
     * Return the country
     *
     * @return BelongsTo
     */
    public function country(): BelongsTo
    {
        return $this->belongsTo(Country::class, 'country_id');
    }

    /**
     * Return the owner
     *
     * @return BelongsTo
     */
    public function owner(): BelongsTo
    {
        return $this->belongsTo(Owner::class, 'owner_id');
    }
}
