# Installation

This section cover all the necessary steps to install and start using the project.

## Before start

The first thing to do is ensure that [Docker](https://www.docker.com/) is installed. Otherwise, is necessary to download and follow the steps inside [Docker website](https://www.docker.com/).

## Backend Environment variables

Once you have all the code in your hands, the first thing that is necessary to do is **rename the .env-sample to .env**

We'll need to fill in some fields before proceeding with the installation process.

| Entry                           | Description                                                                                                                                                                                                                                                                     |
| ------------------------------- | ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- |
| APP_NAME                        | Used when we need to get the application name from the environment variables                                                                                                                                                                                                    |
| APP_DEBUG                       | Define if the backend application (Laravel) will run in debug mode - being verbose with errors                                                                                                                                                                                  |
| APP_KEY                         | Used on encrypter services, need 32 characters random generated                                                                                                                                                                                                                 |
| DATA_PATH_HOST                  | Define the location path where Docker will store the data                                                                                                                                                                                                                       |
| POSTGRES_ENTRYPOINT_INITDB      | After the entrypoint calls initdb to create the default postgres user and database, it will run any _.sql files, run any executable _.sh scripts, and source any non-executable \*.sh scripts found in that directory to do further initialization before starting the service. |
| POSTGRES_PORT                   | Which port PostgreSQL should use (default: 5432)                                                                                                                                                                                                                                |
| POSTGRES_DB                     | The name of the database (PostgreSQL initialization)                                                                                                                                                                                                                            |
| POSTGRES_USER                   | Define the username of database                                                                                                                                                                                                                                                 |
| POSTGRES_PASSWORD               | Define the password of database                                                                                                                                                                                                                                                 |
| DB_CONNECTION                   | Define which database connection Laravel will use (default: pgsql2)                                                                                                                                                                                                             |
| DB_HOST_TEST                    | The host of TEST database (default: postgres)                                                                                                                                                                                                                                   |
| DB_DATABASE_TEST                | The name of TEST database (default: dgp_test)                                                                                                                                                                                                                                   |
| DB_USERNAME_TEST                | The username of TEST database                                                                                                                                                                                                                                                   |
| DB_PASSWORD_TEST                | The password for TEST database                                                                                                                                                                                                                                                  |
| DB2_HOST                        | The host of PROD database (default: postgres)                                                                                                                                                                                                                                   |
| DB2_DATABASE                    | The name of PROD database (default: dgp)                                                                                                                                                                                                                                        |
| DB2_USERNAME                    | The username of PROD database                                                                                                                                                                                                                                                   |
| DB2_PASSWORD                    | The password of PROD database                                                                                                                                                                                                                                                   |
| PAGINATE_PER_PAGE               | Amount of items to retrieve on paginated requests (Default: 10)                                                                                                                                                                                                                 |
| API_DOMAIN                      | Define the API domain to be used on scripts (Default: http://nginx/php)                                                                                                                                                                                                         |
| NETWORKS_DRIVER                 | Define the (network driver)[https://docs.docker.com/network/] to be used inside Docker containers (Default: bridge)                                                                                                                                                             |
| GCP_CLIENT_ID                   | Google Cloud Platform (GCP) application Client Id, used to retrieve access/taxonomy from Google Analytics (GA) accounts                                                                                                                                                         |
| GCP_CLIENT_SECRET               | Google Cloud Platform (GCP) application Client Secret, used to retrieve access/taxonomy from Google Analytics (GA) accounts                                                                                                                                                     |
| YOUTUBE_GCP_KEY                 | Token of Google Cloud Platform (GCP) application created with access to YouTube API                                                                                                                                                                                             |
| TWITTER_CONSUMER_KEY            | Consumer Key created on Twitter Developers to validate Twitter Accounts                                                                                                                                                                                                         |
| TWITTER_CONSUMER_SECRET         | Consumer Secret created on Twitter Developers to validate Twitter Accounts                                                                                                                                                                                                      |
| TWITTER_ACCESS_TOKEN            | Account access token created on Twitter Developers to validate Twitter Accounts                                                                                                                                                                                                 |
| TWITTER_ACCESS_TOKEN_SECRET     | Account secret token created on Twitter Developers to validate Twitter Accounts                                                                                                                                                                                                 |
| SEMRUSH_API_KEY                 | Semrush API Key to retrieve SEO site score                                                                                                                                                                                                                                      |
| FACEBOOK_USER                   | A username (email) of a Facebook Account to evaluate Facebook Pages                                                                                                                                                                                                             |
| FACEBOOK_PASS                   | The password of Facebook Account to evaluate Facebook Pages                                                                                                                                                                                                                     |
| INSTAGRAM_USER_OVERAGE          | A username of a Instagram Account that are above LDA (Legal Drinking Age)                                                                                                                                                                                                       |
| INSTAGRAM_PASS_OVERAGE          | The password of Instragram account above LDA (Legal Drinking Age)                                                                                                                                                                                                               |
| INSTAGRAM_USER_UNDERAGE         | A username of a Instragram Account that are below LDA (Legal Drinking Age)                                                                                                                                                                                                      |
| INSTAGRAM_PASS_UNDERAGE         | The password of a Instagram Account that are below LDA (Legal Drinking Age)                                                                                                                                                                                                     |
| JWT_SECRET                      | A random string used to encrypt JTW authentication tokens                                                                                                                                                                                                                       |
| GCP_CF_LIGHTHOUSE_APAC_REGION   | The region of a GCP Cloud Function (CF) for Lighthouse execution hosted on Asia                                                                                                                                                                                                 |
| GCP_CF_LIGHTHOUSE_APAC_SCRIPT   | The script of a GCP Cloud Function (CF) version 1 for Lighthouse execution hosted on Asia                                                                                                                                                                                       |
| GCP_CF_LIGHTHOUSE_APAC_SCRIPTv2 | The script of a GCP Cloud Function (CF) version 2 for Lighthouse execution hosted on Asia                                                                                                                                                                                       |
| GCP_CF_LIGHTHOUSE_EUR_REGION    | The region of a GCP Cloud Function (CF) for Lighthouse execution hosted on Europe                                                                                                                                                                                               |
| GCP_CF_LIGHTHOUSE_EUR_SCRIPT    | The script of a GCP Cloud Function (CF) version 1 for Lighthouse execution hosted on Europe                                                                                                                                                                                     |
| GCP_CF_LIGHTHOUSE_EUR_SCRIPTv2  | The script of a GCP Cloud Function (CF) version 2 for Lighthouse execution hosted on Europe                                                                                                                                                                                     |
| GCP_CF_LIGHTHOUSE_NAZ_REGION    | The region of a GCP Cloud Function (CF) for Lighthouse execution hosted on North America                                                                                                                                                                                        |
| GCP_CF_LIGHTHOUSE_NAZ_SCRIPT    | The script of a GCP Cloud Function (CF) version 1 for Lighthouse execution hosted on North America                                                                                                                                                                              |
| GCP_CF_LIGHTHOUSE_NAZ_SCRIPTv2  | The script of a GCP Cloud Function (CF) version 2 for Lighthouse execution hosted on North America                                                                                                                                                                              |
| GCP_CREDENTIALS_FILE            | JSON file with authentication details to a service account that have access to GCP (Google Cloud Platform)                                                                                                                                                                      |
| GCP_PROJECT                     | The name of GCP Project                                                                                                                                                                                                                                                         |
| DOMAIN_NAME                     | Production domain name (Default: dgp.ab-inbev.com)                                                                                                                                                                                                                              |
| FILESYSTEM_DRIVER               | specify the default filesystem disk that should be used by backend Framework                                                                                                                                                                                                    |
| ONETRUST_CLIENT_ID              | Client id of application created inside Onetrust to cookie compliant evaluation                                                                                                                                                                                                 |
| ONETRUST_CLIENT_SECRET          | Client secret of application created inside Onetrust to cookie compliant evaluation                                                                                                                                                                                             |
| GCP_COOKIE_CREDENTIALS_FILE     | JSON file with authentication details to a service account that have access to GCP (Google Cloud Platform) - can be the same as GCP_CREDENTIALS_FILE                                                                                                                            |
| TEAMS_WEBHOOK_URL               | Teams Webhook URL to notify after crawler execution                                                                                                                                                                                                                             |
| TEAMS_COOKIE_WEBHOOK_URL        | Teams Webhook URL to notify after cookie crawler execution                                                                                                                                                                                                                      |
| ENV_STAGE                       | Optional for development, on Production environment we need to set this as "prod"                                                                                                                                                                                               |
| NODE_ENV                        | Used to define nodejs environment as test (in this case, we can use headless Chromium even without a serveless environmnent) - default is "test"                                                                                                                                |
| AUTH_FLOW                       | Used to determine which authentication method will be used (can be "ssoABI" or "basic")                                                                                                                                                                                         |
| SSO_CLIENT_ID                   | Needed when AUTH_FLOW = "ssoABI"                                                                                                                                                                                                                                                |
| SSO_CLIENT_SECRET               | Needed when AUTH_FLOW = "ssoABI"                                                                                                                                                                                                                                                |
| SSO_TENANT_ID                   | Needed when AUTH_FLOW = "ssoABI"                                                                                                                                                                                                                                                |
| SSO_REDIRECT_URL                | Needed when AUTH_FLOW = "ssoABI"                                                                                                                                                                                                                                                |
| SSO_AFTER_LOGIN                 | Needed when AUTH_FLOW = "ssoABI"                                                                                                                                                                                                                                                |
| IARD_MODULE                     | Used to enable or disable the IARD informations module on the backend                                                                                                                                                                                                           |
| PRIVACY_MODULE                  | Used to enable or disable the Privacy & Security informations module on the backend                                                                                                                                                                                             |
| EXCELLENCE_MODULE               | Used to enable or disable the Excellence informations module on the backend                                                                                                                                                                                                     |
| DATA_MODULE                     | Used to enable or disable the Data Capture informations module on the backend                                                                                                                                                                                                   |

## Creating containers

After that, we can run

```bash
docker compose up --build -d
```

With this command, Docker will created all the containers and follow the necessary steps to ensure our environment have everything we need.

## Installing backend dependecies

After that, is necessary to install the PHP dependecies inside the PHP container.

For that, we will run

```bash
docker ps
```

To show all the containers that we've created in the previous step.

We'll get something like:

<img :src="$withBase('/images/docker-ps.png#center')" alt="List of all Docker Containers">

Then, we'll execute a command inside PHP container. For that, we use the "Container Id" of PHP (in this image is 25d3281c9d3c):

```bash
docker exec -ti 25d3281c9d3c composer install
```

[Composer](https://getcomposer.org/) is already installed as a global dependency inside PHP Container.

After that, we'll need to install the JavaScript dependencies inside PHP Container:

```bash
docker exec -ti 25d3281c9d3c npm install
```

## Fix permissions

Then, we need to fix the access permisions in our backend. We can solve all our issues with the command below:

```bash
docker exec -ti 25d3281c9d3c chmod -R 777 ./storage
```

::: warning
Remember to change the container id (in this case "25d3281c9d3c") to match with your output of "docker ps"
:::

## Testing

To test if eveyrhint is all right with the backend, you can open you browser and go to the link [http://localhost/php/](http://localhost/php/).

If everything is working, you'll see something like:

<img :src="$withBase('/images/backend-homepage.png#center')" alt="backend homepage">

In case you got something different, is recommended to double check all the previous step. Also, you can set your environment variable (your .env file created on the first step of this document) APP_DEBUG to **true** and you'll get more details about the error you are having.

## Creating database structure

To create database structures, you can use:

```bash
docker exec -ti 25d3281c9d3c php artisan migrate
```

::: warning
Remember to change the container id (in this case "25d3281c9d3c") to match with your output of "docker ps"
:::

## Frontend environment variables

Now, inside the folder "frontend", rename the file **.env-sample** to **.env**.

This files contains the envrionmnet variables to the frontend.

Basically,

| Entry                  | Description                                                                             |
| ---------------------- | --------------------------------------------------------------------------------------- |
| VITE_APIURL            | Define the API url (default: http://localhost/php/api)                                  |
| VITE_ENVIRONMENT       | Define if frontend is running in "dev" or "prod" environmnet                            |
| VITE_AUTH_FLOW         | Used to determine which authentication method will be used (can be "ssoABI" or "basic") |
| VITE_IARD_MODULE       | Used to enable or disable the IARD informations module on the frontend                  |
| VITE_PRIVACY_MODULE    | Used to enable or disable the Privacy & Security informations module on the frontend    |
| VITE_EXCELLENCE_MODULE | Used to enable or disable the Excellence informations module on the frontend            |
| VITE_DATA_MODULE       | Used to enable or disable the Data Capture informations module on the frontend          |
| VITE_DGP_SCORE         | Used to enable or disable the DGP Geral Score informations module on the frontend       |
