# Manual Validation

Once we've finished running our robots, we need to do the manual validation. This process is needed to ensure that we're reporting the correct information to any involved person and be aware of any potential failure of the robot.

We'll validate the information that the robot has provided, we've two different approaches that can be made here:

1. Double-checking the cases that the answer is false: we filter every criterion that the information is "false" and, manually, validate to confirm that the robot data is correct.
2. A complete double-checking, validating everything manually: perform a manual validation on every asset for every criterion.

Normally, we just do the first option, checking the cases where the information is false (or, in rare cases, empty).

Once the manual validation is finished, we update the data in our system. By default, the information that our robot has collected will not be visible public until someone validate this information - we've created a column (**use_on_report**) in our table that stores the results of the tests (**links_criteria**) to define if the information is public or not.

<img :src="$withBase('/images/links-criteria-structure.png#center')" alt="Links-Criteria table structure">

::: details
| Column | Description |
| ------ | ----------- |
| criteria_id | Which criteria this information is related to |
| link_id | The link that this answer belongs to |
| executed_at | Date that the robot started the execution, used to group the answers and generate a report for a specific date |
| response | The information that the robot has got for this link/criteria |
| details | In case that there are any additional details about this answer, like the reason to the link be reproved on this criteria |
| official | Define if this execution is official, and will be considered on the score, or is just a specific execution and should not impact in the final score |
| id | Autoincrement. The identifier of this links_criteria entry |
| use_on_report | Define if this information is public or still needs some validation |
:::

There are two ways to perform the manual validation: 1) Exporting one Spreadsheet file, performing the validation and importing back on the system; 2) Using the interface.

## Spreadsheet Validation

The easier way to get the spreadsheet for a specific validation is exporting the data directly from the database, to help in this process is possible to perform a query in the **reportdgp_draft** materialized view (MV) with the **executed_at** as the condition. Using the MV you get a spreadsheet with all the information grouped by the link in just one line:

<img :src="$withBase('/images/report-dgp-draft-table-structure.png#center')" alt="Links-Criteria table structure">

Once you've finished the manual validation, and updating the spreadsheet data, you need to import this data back into the platform. We've created a script that will parse all the spreadsheet data and insert it into the database - the script is available [here](https://dgp.ab-inbev.com/php/import-script.zip).

To use this script, you'll need to update the **credentials** variable to be the same from your environment - possibly this information will be the same as you've defined in the [installation process](https://dgp.ab-inbev.com/php/docs/docs/getting-started/installation.html#backend-environment-variables).

```javascript
const credentials = {
  user: 'psql',
  host: 'localhost',
  database: 'dgp',
  password: 'password',
  port: 5432,
};
```

Also, you need to ensure that your spreadsheet file is following the template inside the "excel" folder (the column name needs to be the same as the criteria) and is included in the array to be handled by the script:

```javascript
const files = ['./excel/20221229.xlsx'];
```

::: tip
Consider deleting from the database the older information, even not public will be using your storage space.
:::

## Using the interface

We've created one page, accessible by any admin user:

<img :src="$withBase('/images/manual-validation-interface.png#center')" alt="Manual validation interface">

Once the user clicks on an entry on this screen, will have the information from the previous execution on the side of the current execution - this can help to discover any asset that has changed or some problem with the robot.

This method allows any user to perform manual validation, not needing any technical background to update the configuration with the database, delete old data, etc.
