# Asset Management

There are two ways to manage the assets: directly in the database or using the importer that we've created.

## Database

The first option to perform the management operations (create, update or delete) in the assets is using the database directly.

All the information is stored in a table called **links**, with the structure below:

<img :src="$withBase('/images/links-table.png#center')" alt="Table Links structure">

::: details
| Column | Description |
-------- | ----------- |
| id | Autoincrement, used to identify the link |
| link | The link that will be audited |
| additional_crawler | Define the way the robots can bypass agegate validation |
| semrush_project_id | Used to retrieve the information from Semrush API |
| brand_id | The identifier (id) of the brand that this assets belongs to |
| channel_id | The identifier (id) of the channel (site, facebook, twitter, youtube or instagram) |
| country_id | The identifier (id) of the country that this assets belongs to |
| owner_id | The identifier (id) of the owner that this assets belongs to |
| http_status | Used to define if the website is "live" or not |
| business_area | The business area of the asset |
| environment | Define if the asset is in Production, Development, Staging |
| consider | Define if asset will be considered in the evaluations or not |
| scope | The scope of the asset |
| in_ga | Define if the website were discovered based on Google Analytics properties list |
| in_acquia | Define if the website is hosted by Acquia |
| in_bp | Define if the website were discovered by Brand Protection Team |
| in_cdp | Define if the website were discovered based on CDP properties list |
| alcoholic | Define if the asset is alcoholic or not |
:::

::: tip
The robots will always get the information from this table and audit everything that the field **consider** is **true**.
:::

## Importer

The other option to manage the assets is using the importer, a more intuitive and friendly way to manage the assets in batches.

<img :src="$withBase('/images/asset-importer.png#center')" alt="Asset Importer">

Any user with admin privileges can use this importer to update the asset list.

There is a template with all the needed information available [here](https://dgp.ab-inbev.com/php/asset-sheet-import.xlsx), and below there is the mapping between the importer and the sample provided:

| Importer Field       | Excel Column | Description                                                                             |
| -------------------- | :----------: | --------------------------------------------------------------------------------------- |
| Domain Column        |      A       | The link that will be considered                                                        |
| Country Column       |      B       | The country that the asset belongs to                                                   |
| Brand Column         |      C       | The brand that the asset belongs to                                                     |
| Consider Column      |      D       | The consider columnd, that define it asset should be audited or not                     |
| Environment Column   |      E       | Define if the asset is in Production, Development, Staging                              |
| Status Columns       |      F       | Define if the website is "live" or not                                                  |
| Scope Column         |      G       | The scope of the asset                                                                  |
| Business Area Column |      G       | The business area of the asset                                                          |
| Asset Type Column    |      H       | Define the channel of the asset                                                         |
| Assessment Column    |      I       | Define the status of PIA (when Select Criteria = "PIA Completion")                      |
| Assessment Column    |      J       | Define the status of Risk Rate (when Select Criteria = "Risk Rate")                     |
| Assessment Column    |      N       | Define the status of Vulnerability Level (when Select Criteria = "Vulnerability Level") |

::: tip
Keep in mind that when uploading a new file, all the assets will have the **consider** set to **false** - and only the assets that are included in the file will have the **consider** field updated to **true** after the importer process.

Also, remember that only assets that **consider** is **true** will be audited.
:::
