import axios from 'axios';

function getCookie(name) {
  const value = `; ${document.cookie}`;
  const parts = value.split(`; ${name}=`);
  if (parts.length === 2) return parts.pop().split(';').shift();
}

const api = axios.create({
  baseURL: import.meta.env.VITE_APIURL,
  headers: {
    Accept: 'application/json',
    'Content-Type': 'application/json',
  },
});

api.interceptors.request.use(
  (config) => {
    const ssoExpires = getCookie('sso_expires')
      ? getCookie('sso_expires')
      : localStorage.getItem('sso_expires') || '';
    const ssoSessionId = getCookie('session_identifier')
      ? getCookie('session_identifier')
      : localStorage.getItem('session_identifier') || '';

    if (ssoExpires && ssoSessionId) {
      config.headers.ssoExpires = ssoExpires;
      config.headers.ssoToken = ssoSessionId;
    }

    return config;
  },
  (error) => {
    return Promise.reject(error);
  }
);

api.interceptors.response.use(
  (config) => {
    return config;
  },
  (error) => {
    if (error.response.status === 401) {
      localStorage.removeItem('session_identifier');
      localStorage.removeItem('sso_expires');
      localStorage.removeItem('user_id');
      document.cookie = 'session_identifier=; Max-Age=-99999999;';
      document.cookie = 'sso_expires=; Max-Age=-99999999;';
      window.location = '/auth/login';
    }
    return Promise.reject(error);
  }
);

export default api;
