import { createApp } from 'vue';
import App from './App.vue';
import { VuesticPlugin } from 'vuestic-ui';
import 'vuestic-ui/dist/vuestic-ui.css';
import router from './router/index';
import store from './store';
import utils from './utils/utils';
import VueApexCharts from 'vue3-apexcharts';
import { createGtm } from '@gtm-support/vue-gtm';

import api from './helpers/axios';

const app = createApp(App);

app.config.globalProperties.apiUrl = import.meta.env.VITE_APIURL;

app.config.globalProperties.ssoClientId = import.meta.env.VITE_SSO_CLIENT_ID;
app.config.globalProperties.ssoTenantId = import.meta.env.VITE_SSO_TENANT_ID;
app.config.globalProperties.ssoRedirectUrl =
  import.meta.env.VITE_SSO_REDIRECT_URL;

app.config.globalProperties.authFlow = import.meta.env.VITE_AUTH_FLOW || '';

app.config.globalProperties.iardModule = import.meta.env.VITE_IARD_MODULE || 0;
app.config.globalProperties.privacyModule =
  import.meta.env.VITE_PRIVACY_MODULE || 0;
app.config.globalProperties.excellenceModule =
  import.meta.env.VITE_EXCELLENCE_MODULE || 0;
app.config.globalProperties.dataModule = import.meta.env.VITE_DATA_MODULE || 0;
app.config.globalProperties.geralScore = import.meta.env.VITE_DGP_SCORE || 0;

app.config.globalProperties.$utils = utils;

app.config.globalProperties.api = api;

app.use(VuesticPlugin, {
  colors: {
    primary: '#f5e003',
    secondary: '#000000',
  },
});

app.use(router);
app.use(store);
app.component('ApexCharts', VueApexCharts);

if (import.meta.env.VITE_ENVIRONMENT !== 'dev') {
  app.use(
    createGtm({
      id: 'GTM-TG53DP5',
      enabled: true,
      loadScript: true,
      vueRouter: router,
    })
  );
}

app.mount('#app');
