<?php

use App\Http\Controllers\AgegateDataValidation;
use App\Http\Controllers\ApiTokenController;
use App\Http\Controllers\Auth\UserAuthController;
use App\Http\Controllers\BrandController;
use App\Http\Controllers\ChannelController;
use App\Http\Controllers\CommonController;
use App\Http\Controllers\CookieReportController;
use App\Http\Controllers\CountryController;
use App\Http\Controllers\CriteriaController;
use App\Http\Controllers\ImportBpController;
use App\Http\Controllers\KeywordsController;
use App\Http\Controllers\LinkCriteriaController;
use App\Http\Controllers\LinksController;
use App\Http\Controllers\LocaleController;
use App\Http\Controllers\ManualValidationController;
use App\Http\Controllers\NpsFeedbackController;
use App\Http\Controllers\TicketsController;
use App\Http\Controllers\UserController;
use App\Http\Controllers\ZoneController;
use App\Http\Middleware\UsersTokenValidate;
use App\Http\Middleware\ValidateAdminUser;
use App\Http\Middleware\ValidateApiLogin;
use App\Http\Middleware\ValidateSuperAdminUser;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Route;

/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
|
| Here is where you can register API routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| is assigned the "api" middleware group. Enjoy building your API!
|
*/

Route::middleware('auth:api')->get('/user', function (Request $request) {
    return $request->user();
});

Route::get('channel', function () {
    return app(ChannelController::class)->index();
});

Route::get('channel/{id}', function (int $id) {
    return app(ChannelController::class)->show($id);
});

Route::get('consider', function () {
    return app(LinksController::class)->indexConsider();
});

Route::get('country', function () {
    return app(CountryController::class)->index();
});

Route::get('country/{id}', function (int $id) {
    return app(CountryController::class)->show($id);
});

Route::get('link', function () {
    return app(LinksController::class)->index();
});

Route::get('link/{id}', function (int $id) {
    return app(LinksController::class)->show($id);
});

Route::get('link-criteria/{id}', function (int $id) {
    return app(LinkCriteriaController::class)->show($id);
});

Route::post('link', function (Request $request) {
    return app(LinksController::class)->store($request->getContent());
});

Route::post('link/{id}', function (int $id, Request $request) {
    return app(LinksController::class)->update($id, $request->getContent());
});

Route::get('link-criteria/execution/{executed_at}', function (string $executedAt) {
    return app(LinkCriteriaController::class)->getTestsByExecution($executedAt);
});

Route::post('link-criteria', function (Request $request) {
    return app(LinkCriteriaController::class)->store($request->getContent());
});

Route::get('zone', function () {
    return app(ZoneController::class)->index();
});

Route::get('zone/{id}', function (int $id) {
    return app(ZoneController::class)->show($id);
});

Route::match(array('GET', 'POST'), '/auth/validate-sso-token', function (Request $request) {
    return app(UserAuthController::class)->sso_ValidateToken($request);
});

Route::get('/login/sso/', function (Request $request) {
    return app(UserAuthController::class)->sso_login($request);
});

Route::post('/agegate-data-validation', function (Request $request) {
    return app(AgegateDataValidation::class)->store($request);
});

Route::post('/manual-validation', function (Request $request) {
    return app(ManualValidationController::class)->storeManualValidation($request->getContent());
});

Route::middleware([ValidateApiLogin::class])->group(function () {
    //___Common
    Route::get('/filters-data/{page?}', function (Request $request) {
        return app(CommonController::class)->filtersData($request);
    });

    Route::get('home', function (Request $request) {
        return app(CommonController::class)->homePage($request);
    });

    Route::get('results-by-web-property', function (Request $request) {
        return app(CommonController::class)->resultsByWebProp($request);
    });

    Route::get('results-by-web-property/export', function (Request $request) {
        return app(CommonController::class)->resultsByWebPropExport($request);
    });

    Route::get('results-by-country', function (Request $request) {
        return app(CommonController::class)->resultsByCountry($request);
    });

    Route::get('results-by-country/export', function (Request $request) {
        return app(CommonController::class)->resultsByCountryExport($request);
    });

    Route::get('web-properties-list', function (Request $request) {
        return app(LinksController::class)->webPropertiesList($request);
    });

    Route::get('web-properties-list/export', function (Request $request) {
        return app(LinksController::class)->exportWebPropertiesList($request);
    });

    Route::get('reports-new/{type}', function (string $type, Request $request) {
        return app(CommonController::class)->reportsData($type, $request);
    });

    Route::get('reports-new/export/{type}', function (string $type, Request $request) {
        return app(CommonController::class)->reportsDataExport($type, $request);
    });

    Route::get('common/criterias-list', function () {
        return app(CriteriaController::class)->getCriteriasList();
    });

    Route::post('user-feedback', function (Request $request) {
        return app(NpsFeedbackController::class)->userFeedback($request);
    });

    Route::get('get-last-feedback-by-user', function (Request $request) {
        return app(NpsFeedbackController::class)->getLastFeedbackByUser($request);
    });
    //___Common

    //___Keywords
    Route::get('keywords', function (Request $request) {
        return app(KeywordsController::class)->getKeywordsPaginated($request);
    });

    Route::get('keywords/export', function (Request $request) {
        return app(KeywordsController::class)->exportKeywords($request);
    });

    Route::get('keywords/filters-data', function () {
        return app(KeywordsController::class)->filterKeywords();
    });

    Route::get('get-keyword-data/{id}', function (int $id) {
        return app(KeywordsController::class)->showDetailedKeyword($id);
    });
    //___Keywords

    //___Locale
    Route::get('locale', function (Request $request) {
        return app(LocaleController::class)->index_paginated($request);
    });

    // Route::get('locale/{id}', function (int $id) {
    //     return app(LocaleController::class)->show($id);
    // });

    Route::post('locale', function (Request $request) {
        return app(LocaleController::class)->store($request->getContent());
    });

    Route::post('locale/{id}', function (int $id, Request $request) {
        return app(LocaleController::class)->update($id, $request->getContent());
    });

    Route::delete('locale/{id}', function (int $id) {
        return app(LocaleController::class)->destroy($id);
    });
    //___Locale

    Route::get('lighthouse-latest', function () {
        return app(LinkCriteriaController::class)->getLatestLighthouse();
    });

    Route::get('lighthouse-report/{link}/{executedAt}', function (string $link, string $executedAt) {
        return app(LinkCriteriaController::class)->getLighthouseReport($link, $executedAt);
    });

    //___Brands
    Route::get('brand', function (Request $request) {
        return app(BrandController::class)->index_paginated($request);
    });

    // Route::get('brand/{id}', function (int $id) {
    //     return app(BrandController::class)->show($id);
    // });

    Route::delete('brand/{id}', function (int $id) {
        return app(BrandController::class)->destroy($id);
    });

    Route::post('brand', function (Request $request) {
        return app(BrandController::class)->store($request->getContent());
    });

    Route::post('brand/{id}', function (int $id, Request $request) {
        return app(BrandController::class)->update($id, $request->getContent());
    });
    //___Brands

    //___Others
    Route::post('create-asset-details-ticket', function (Request $request) {
        return app(TicketsController::class)->createAssetDetailsTicket($request);
    });

    Route::get('asset-details-ticket-info/{id}', function (int $id) {
        return app(TicketsController::class)->getAssetDetailsTicketInfo($id);
    });

    Route::post('create-disprove-result-ticket', function (Request $request) {
        return app(TicketsController::class)->createDisproveResultTicket($request);
    });

    Route::post('create-keyword-ticket', function (Request $request) {
        return app(TicketsController::class)->createKeywordsTicket($request);
    });

    Route::post('create-ownership-ticket', function (Request $request) {
        return app(TicketsController::class)->createOwnershipTicket($request);
    });

    Route::get('get-user-tickets', function (Request $request) {
        return app(TicketsController::class)->getUserTickets($request);
    });

    Route::get('api-token', function (Request $request) {
        return app(ApiTokenController::class)->retrieve($request);
    });

    Route::post('api-token', function (Request $request) {
        return app(ApiTokenController::class)->generate($request);
    });

    Route::get('ownership-details/{id}', function (int $id) {
        return app(CommonController::class)->getOwnershipData($id);
    });
    //___Others

    //__Cookie reports
    Route::get('cookie-domain-level', function (Request $request) {
        return app(CookieReportController::class)->domainLevelReport($request);
    });

    Route::get('cookie-get-filter', function () {
        return app(CookieReportController::class)->getCookieFilterData();
    });

    Route::get('cookie-export-report', function (Request $request) {
        return app(CookieReportController::class)->exportReport($request);
    });

    Route::get('cookie-export-list', function (Request $request) {
        return app(CookieReportController::class)->exportWebsiteCookies($request);
    });
    //__Cookie reports
});

Route::middleware([ValidateApiLogin::class, ValidateAdminUser::class])->group(function () {
    Route::get('get-tickets', function (Request $request) {
        return app(TicketsController::class)->getTickets($request);
    });
    Route::post('update-ticket-result', function (Request $request) {
        return app(TicketsController::class)->updateTicketResult($request);
    });
    Route::post('update-ticket-ownership', function (Request $request) {
        return app(TicketsController::class)->updateTicketOwnership($request);
    });
    Route::post('update-keywords-ticket-result', function (Request $request) {
        return app(TicketsController::class)->updateKeywordsTicketResult($request);
    });
    Route::get('get-users-filter', function () {
        return app(UserController::class)->getUsersToFilter();
    });
});

Route::middleware([ValidateApiLogin::class, ValidateSuperAdminUser::class])->group(function () {
    Route::post('import-bp', function (Request $request) {
        return app(ImportBpController::class)->importBpHandler($request);
    });
    Route::post('get-spreadsheet-data', function (Request $request) {
        return app(ImportBpController::class)->saveSpreadsheet($request);
    });

    Route::get('api-token', function (Request $request) {
        return app(ApiTokenController::class)->retrieve($request);
    });

    Route::post('api-token', function (Request $request) {
        return app(ApiTokenController::class)->generate($request);
    });

    Route::get('/get-asset-details-tickets/export', function (Request $request) {
        return app(TicketsController::class)->exportAssetDetailsTickets($request);
    });

    Route::get('get-users', function (Request $request) {
        return app(UserController::class)->getUsers($request);
    });

    Route::post('update-user-data', function (Request $request) {
        return app(UserController::class)->updateUserData($request);
    });

    Route::get('get-feedbacks', function (Request $request) {
        return app(NpsFeedbackController::class)->getFeedbacks($request);
    });

    Route::get('export-feedbacks', function (Request $request) {
        return app(NpsFeedbackController::class)->exportFeedbacks($request);
    });

    Route::get('/manual-validation', function (Request $request) {
        return app(ManualValidationController::class)->getExecutionData($request);
    });

    Route::get('/manual-validation/{id}', function (int $id, Request $request) {
        return app(ManualValidationController::class)->getAssetExecutionData($id, $request);
    });

    Route::post('/manual-validation/display-on-report', function (Request $request) {
        return app(ManualValidationController::class)->assetDisplayOnReport($request);
    });

    Route::post('/manual-validation/validation', function (Request $request) {
        return app(ManualValidationController::class)->assetValidation($request);
    });

    Route::post('/manual-validation/display-on-report/bulk', function (Request $request) {
        return app(ManualValidationController::class)->bulkAssetDisplayOnReport($request);
    });

    Route::post('/manual-validation/validation/bulk', function (Request $request) {
        return app(ManualValidationController::class)->bulkAssetValidation($request);
    });

    Route::post('/manual-validation/refresh-mvs', function (Request $request) {
        return app(ManualValidationController::class)->refreshMvs($request);
    });
});

Route::middleware([UsersTokenValidate::class])->group(function () {
    Route::get('lighthouse/{zone?}/{country?}/{link?}/{executedAt?}', function (Request $request) {
        return app(LinkCriteriaController::class)->getLighthouseData($request);
    });

    Route::get('cookie-information/{link}/{executedAt?}', function (Request $request) {
        return app(CookieReportController::class)->getCookieRelatedData($request);
    });
});

Route::post('/basic/register', function (Request $request) {
    return app(UserController::class)->store($request->getContent());
});

Route::post('/basic/login', function (Request $request) {
    return app(UserController::class)->login($request->getContent());
});
